/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:net.bigeon.gclc.command.ParametrizedCommand.java
 * Created on: Dec 24, 2014
 */
package net.bigeon.gclc.command;

/*-
 * #%L
 * Generic Command Ligne console
 * %%
 * Copyright (C) 2014 - 2018 bigeon.fr
 * %%
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * #L%
 */
import java.io.IOException;
import java.util.Set;

import net.bigeon.gclc.exception.CommandRunException;
import net.bigeon.gclc.exception.InvalidParameterException;
import net.bigeon.gclc.manager.ConsoleInput;
import net.bigeon.gclc.manager.ConsoleOutput;

/** A command relying on the {@link CommandParameters} to store parameters
 * values.
 *
 * @author Emmanuel BIGEON */
public abstract class ParametrizedCommand extends Command {

    private final ParametrizedCommandData data;

    /** Create a parametrized command.
     * <p>
     * Implementation are supposed to call the {@link #addBooleanParameter(String)}
     * and {@link #addStringParameter(String, boolean)} method to set the
     * parameters.
     *
     * @param name the name */
    public ParametrizedCommand(final String name) {
        this(name, true);
    }

    /** Create a parametrized command.
     * <p>
     * Implementation are supposed to call the {@link #addBooleanParameter(String)}
     * and {@link #addStringParameter(String, boolean)} method to set the
     * parameters.
     *
     * @param name the name
     * @param strict if the arguments are restricted to the declared ones */
    public ParametrizedCommand(final String name, final boolean strict) {
        super(name);
        data = new ParametrizedCommandData(strict);
    }

    /** Add a boolean parameter to defined parmaters.
     *
     * @param flag the boolean flag
     * @throws InvalidParameterException if the parameter is already defined as a
     *         string parameter */
    protected final void addBooleanParameter(final String flag) {
        data.addBooleanParameter(flag);
    }

    /** Add a string parameter to defined parmaters.
     *
     * @param flag the parameter flag
     * @param needed if the parameter's absence should cause an exception
     * @throws InvalidParameterException if the parameter is already defined as a
     *         boolean parameter */
    protected final void addStringParameter(final String flag, final boolean needed) {
        data.addStringParameter(flag, needed);
    }

    /** Actually performs the execution after parsing the parameters.
     *
     * @param out the output
     * @param in the input
     * @param parameters the command parameters
     * @throws CommandRunException if the command failed */
    protected abstract void doExecute(ConsoleOutput out, ConsoleInput in,
            CommandParameters parameters) throws CommandRunException;

    /* (non-Javadoc)
     * @see net.bigeon.gclc.command.Command#execute(java.lang.String[]) */
    @Override
    public final void execute(final ConsoleOutput output, final ConsoleInput input,
            final String... args) throws CommandRunException {
        try {
            doExecute(output, input, data.getParameters(input, args));
        } catch (final IOException e) {
            throw new CommandRunException("Unable to get parameters", e);
        }
    }

    /** Retrieve the boolean parameters (aka flags).
     *
     * @return the set of boolean parameters */
    public final Set<String> getBooleanParameters() {
        return data.getBooleanParameters();
    }

    /** Retrieve the parameter names.
     *
     * @return the stringParams */
    public final Set<String> getParameters() {
        return data.getParameters();
    }

    /** Get the string parameters names.
     *
     * @return the stringParams */
    public final Set<String> getStringParameters() {
        return data.getStringParameters();
    }

    /** Test if a parameter is needed.
     *
     * @param param the parameter name
     * @return if the parameter is needed */
    public final boolean isNeeded(final String param) {
        return data.isNeeded(param);
    }

    /** If the command refuse unrecognized parameters.
     *
     * @return the strict */
    public final boolean isStrict() {
        return data.isStrict();
    }
}
